function features = feature_extract_pitch(GESTURE_ID)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  This script extracts relevant features from the time series for
%  each sensor.
%
%  Data is stored in .mat files with a data structure for each
%  sensor.  Each structure has the following syntax: 
%
%  sensor.LO_SAMP
%  sensor.LO_X
%  sensor.LO_Y
%  sensor.LO_Z
%  sensor.LO_PI
%  sensor.LO_RO
%  sensor.LO_YA
%  sensor.HI_SAMP
%  sensor.HI_X
%  sensor.HI_Y
%  sensor.HI_Z
%  sensor.HI_PI
%  sensor.HI_RO
%  sensor.HI_YA
%
%  where sensor can be hand, forearm, upper_arm, chest, or waist.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[hand,forearm,upper_arm,chest,waist] = getData(GESTURE_ID);

% Find zero point for peak measurements

[val_ref,ind_ref] = max(abs(hand.HI_Z));

features = [];

% Hand

% Find magnitude of vectors:
h_lin_acc = sqrt(hand.HI_X.^2 + hand.HI_Y.^2 + hand.HI_Z.^2);
h_rot_vel = sqrt(hand.HI_RO.^2 + hand.HI_PI.^2 + hand.HI_YA.^2); 

% Find peak vlaues,times:
[h_val_acc,h_ind_acc] = max(abs(h_lin_acc));   % remember to synch to sample
[h_val_rot,h_ind_rot] = max(abs(h_rot_vel)); 

h_ind_acc = h_ind_acc-ind_ref;
h_ind_rot = h_ind_rot-ind_ref;

% Find FWHM

h_half_ind_acc = find(abs(h_lin_acc) > 0.5*h_val_acc);
h_fwhm_acc     = h_half_ind_acc(end)-h_half_ind_acc(1);

h_half_ind_rot = find(abs(h_rot_vel) > 0.5*h_val_rot);
h_fwhm_rot     = h_half_ind_rot(end)-h_half_ind_rot(1);

% Find total energy in peak
h_e_acc = sum(h_lin_acc.^2);
h_e_rot = sum(h_rot_vel.^2);

features = cat(1,features,h_val_acc,h_val_rot,h_ind_acc,h_ind_rot, ...
               h_fwhm_acc,h_fwhm_rot,h_e_acc,h_e_rot);

% Forearm

% Find magnitude of vectors:
f_lin_acc = sqrt(forearm.HI_X.^2 + forearm.HI_Y.^2 + forearm.HI_Z.^2);
f_rot_vel = sqrt(forearm.HI_RO.^2 + forearm.HI_PI.^2 + forearm.HI_YA.^2); 

% Find peak vlaues,times:
[f_val_acc,f_ind_acc] = max(abs(f_lin_acc));   % remember to synch to sample
[f_val_rot,f_ind_rot] = max(abs(f_rot_vel)); 

f_ind_acc = f_ind_acc-ind_ref;
f_ind_rot = f_ind_rot-ind_ref;

% Find FWHM

f_half_ind_acc = find(abs(f_lin_acc) > 0.5*f_val_acc);
f_fwhm_acc     = f_half_ind_acc(end)-f_half_ind_acc(1);

f_half_ind_rot = find(abs(f_rot_vel) > 0.5*f_val_rot);
f_fwhm_rot     = f_half_ind_rot(end)-f_half_ind_rot(1);

% Find total energy in peak
f_e_acc = sum(f_lin_acc.^2);
f_e_rot = sum(f_rot_vel.^2);

features = cat(1,features,f_val_acc,f_val_rot,f_ind_acc,f_ind_rot, ...
               f_fwhm_acc,f_fwhm_rot,f_e_acc,f_e_rot);

% Upper Arm

% Find magnitude of vectors:
u_lin_acc = sqrt(upper_arm.HI_X.^2 + upper_arm.HI_Y.^2 + upper_arm.HI_Z.^2);
u_rot_vel = sqrt(upper_arm.HI_RO.^2 + upper_arm.HI_PI.^2 + upper_arm.HI_YA.^2); 

% Find peak vlaues,times:
[u_val_acc,u_ind_acc] = max(abs(u_lin_acc));   % remember to synch to sample
[u_val_rot,u_ind_rot] = max(abs(u_rot_vel)); 

u_ind_acc = u_ind_acc-ind_ref;
u_ind_rot = u_ind_rot-ind_ref;

% Find FWHM

u_half_ind_acc = find(abs(u_lin_acc) > 0.5*u_val_acc);
u_fwhm_acc     = u_half_ind_acc(end)-u_half_ind_acc(1);

u_half_ind_rot = find(abs(u_rot_vel) > 0.5*u_val_rot);
u_fwhm_rot     = u_half_ind_rot(end)-u_half_ind_rot(1);

% Find total energy in peak
u_e_acc = sum(u_lin_acc.^2);
u_e_rot = sum(u_rot_vel.^2);

features = cat(1,features,u_val_acc,u_val_rot,u_ind_acc,u_ind_rot, ...
               u_fwhm_acc,u_fwhm_rot,u_e_acc,u_e_rot);

% Chest

% Find magnitude of vectors:
c_lin_acc = sqrt(chest.HI_X.^2 + chest.HI_Y.^2 + chest.HI_Z.^2);
c_rot_vel = sqrt(chest.HI_RO.^2 + chest.HI_PI.^2 + chest.HI_YA.^2); 

% Find peak vlaues,times:
[c_val_acc,c_ind_acc] = max(abs(c_lin_acc));   % remember to synch to sample
[c_val_rot,c_ind_rot] = max(abs(c_rot_vel)); 

c_ind_acc = c_ind_acc-ind_ref;
c_ind_rot = c_ind_rot-ind_ref;

% Find FWHM

c_half_ind_acc = find(abs(c_lin_acc) > 0.5*c_val_acc);
c_fwhm_acc     = c_half_ind_acc(end)-c_half_ind_acc(1);

c_half_ind_rot = find(abs(c_rot_vel) > 0.5*c_val_rot);
c_fwhm_rot     = c_half_ind_rot(end)-c_half_ind_rot(1);

% Find total energy in peak
c_e_acc = sum(c_lin_acc.^2);
c_e_rot = sum(c_rot_vel.^2);

features = cat(1,features,c_val_acc,c_val_rot,c_ind_acc,c_ind_rot, ...
               c_fwhm_acc,c_fwhm_rot,c_e_acc,c_e_rot);

% Waist

% Find magnitude of vectors:
w_lin_acc = sqrt(waist.HI_X.^2 + waist.HI_Y.^2 + waist.HI_Z.^2);
w_rot_vel = sqrt(waist.HI_RO.^2 + waist.HI_PI.^2 + waist.HI_YA.^2); 

% Find peak vlaues,times:
[w_val_acc,w_ind_acc] = max(abs(w_lin_acc));   % remember to synch to sample
[w_val_rot,w_ind_rot] = max(abs(w_rot_vel)); 

w_ind_acc = w_ind_acc-ind_ref;
w_ind_rot = w_ind_rot-ind_ref;

% Find FWHM

w_half_ind_acc = find(abs(w_lin_acc) > 0.5*w_val_acc);
w_fwhm_acc     = w_half_ind_acc(end)-w_half_ind_acc(1);

w_half_ind_rot = find(abs(w_rot_vel) > 0.5*w_val_rot);
w_fwhm_rot     = w_half_ind_rot(end)-w_half_ind_rot(1);

% Find total energy in peak
w_e_acc = sum(w_lin_acc.^2);
w_e_rot = sum(w_rot_vel.^2);

features = cat(1,features,w_val_acc,w_val_rot,w_ind_acc,w_ind_rot, ...
               w_fwhm_acc,w_fwhm_rot,w_e_acc,w_e_rot);
